/** 
 @file md-sr.js
 
 @brief This file contain implementation of scene responder setup methods.

 @author: Gregor

 @date: 17.06.2013alarm
*/


/**
    Global variable defines if SR should force the action (true) or not (false)
*/
var MD_SR_FORCE = true;
/** Global variable defines if SR is visible in the SR UI:
    - empty or "system-addon-scene-responder", visible
    - otherwise not visible */
var MD_UI_SCOPE = "motion-detector"; // the name must be exact as app name

var APP_ID = "motion-detector";

/** 
 @brief Creates motion state name on passed room ID
 
 @param nRoomID - room ID
  
 @return State name
*/
function MD_buildRoomMotionStateName(nRoomID) {
    return "zone."+ nRoomID + ".motion"; 
}


/** 
 @brief Creates light state name on passed room ID
 
 @param nRoomID - room ID
  
 @return State name
*/
function MD_buildRoomLightStateName(nRoomID) {
    return "zone."+ nRoomID + ".light"; 
}


/** 
 @brief Creates manual switch status state name on passed room ID
 
 @param nRoomID - room ID
  
 @return State name
*/
function MD_buildRoomManualSwitchStateName(nRoomID) {
    return "manual_switch_state_"+ nRoomID;
}


/** 
 @brief Creates fade off status state name on passed room ID
 

 @param nRoomID - room ID
  
 @return State name
*/
function MD_buildRoomFadeOffStateName(nRoomID) {
    return "fade_off_state_"+ nRoomID;
}


/** 
 @brief Creates final off status state name on passed room ID
 
 @param nRoomID - room ID
  
 @return State name
*/
function MD_buildRoomFinalOffStateName(nRoomID) {
    return "final_off_state_"+ nRoomID;
}


/**
 @brief Configure the FINAL DELAY OFF SR for specific room
 

 @param nRoomID - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nDelay  - delay in seconds  

 @return FINAL DELAY OFF SR is created, updated or deleted
*/
function MD_setupFinalOffDelaySR(nRoomID, nDelay) {
    try {
        lInfo.logln("MD_setupFinalOffDelaySR for roomID: " + nRoomID);

        if (nDelay > 0) {
            var system_states_final_off_delay = [];

            var room_manual_switch_state_name = MD_buildRoomManualSwitchStateName(nRoomID);
            var room_light_state_name = MD_buildRoomLightStateName(nRoomID);
            var room_fade_off_state_name = MD_buildRoomFadeOffStateName(nRoomID);
            var room_final_off_state_name = MD_buildRoomFinalOffStateName(nRoomID);

            //system_states_auto_off.push({"name" : "presence", "value" : "present"});
            
            // For delay SR has trigger manual switch ON
            var triggers_final_off_delay = [
                {
                    "type" : "addon-state-change",
                    "name" : room_fade_off_state_name,
                    "state" : "active", // "1" // "active"
                    "addonId": "motion-detector"
                }
            ];
                            
            var actions_final_off_delay = [
                {
                    "type" : "change-addon-state",
                    "statename" : room_final_off_state_name,
                    "state" : "active",
                    "delay" : 0,
                    "addonId" : "motion-detector",
                    "category" : "manual"
                }
            ];
                                 
            // lights can be turned OFF only if NO manual switch
            var addon_states_final_off_delay = [
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            ];
                            
            // Create a delay SR for the final off action
            var srFinalOffDelay = {
                "name" : "md_" + nRoomID + "_final_off_delay",
                "id" : "md_" + nRoomID + "_final_off_delay",
                "delay" : nDelay,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "singularTriggered": true,
                "triggers" : triggers_final_off_delay,
                "actions" : actions_final_off_delay,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : null,
                    "systemState": system_states_final_off_delay,
                    "addonStates": {
                        "motion-detector": addon_states_final_off_delay
                    }
                }
            };
            
            var evt = new Event("system-addon-scene-responder.config"
                                , { actions:'save'
                                    , value:escape(JSON.stringify(srFinalOffDelay))
                                  }
            );
            evt.raise();
            lInfo.logln("Created SR Delay md_" + nRoomID +"_final_off_delay");
        }
    } catch (e) {
        lWarn.logln("Error by saving MD_setupFinalOffDelaySR: " + e);
        for (var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
}


/**

 @brief Configure the ALARM SR for specific room
 
 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bAlarmMotion        - true if also alarm functionality is supported, otherwise false
  
 @return 2 ALARM SR (set alarm, undo alarm) are created, updated or deleted
*/
function MD_setupAlarmSR(nRoomID, nRoomModeID, bAlarmMotion) {
    try {
        lInfo.logln("MD_setupAlarmSR for roomID: " + nRoomID + ", modeID:" + nRoomModeID);

        var room_motion_state_name = MD_buildRoomMotionStateName(nRoomID);

        if ((bAlarmMotion) && (nRoomModeID != 4) && (nRoomModeID != 5)) {
            var system_states_alarm = [];
            var system_states_undo_alarm = [];
            var triggers_alarm = [];
            var triggers_undo_alarm = [];

            system_states_alarm.push({"name" : "presence", "value" : "absent"});
            system_states_undo_alarm.push({"name" : "presence", "value" : "absent"});

            system_states_alarm.push(
                {
                    "name" : "alarm", 
                    "value" : "2" // inactive // false
                }
            );

            var room_light_state_name = MD_buildRoomLightStateName(nRoomID);

            triggers_alarm = [
                {
                    "type" : "state-change",
                    "state": "active",
                    "name" : room_motion_state_name
                }
            ];

            triggers_undo_alarm = [
                {
                    "type" : "state-change",
                    "state": "inactive",
                    "name" : room_motion_state_name
                }
            ];

            // Alarm mode is ON, create new SR to signal the alarm
            lInfo.logln("alarm mode is ON");
            // Create alarm SR
            srAlarm = {
                "name" : "md_" + nRoomID + "_alarm",
                "id" : "md_" + nRoomID + "_alarm",
                "delay" : 0,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers" : triggers_alarm,                                          
                "actions" : [{
                    "type" : "zone-scene",
                    "zone" : 0, // broadcast
                    "group" : 0,
                    "scene" : 74, // alarm signal
                    "force" : MD_SR_FORCE,
                    "delay" : 0,
                    "category" : "manual"           
                }],
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : null,
                    "systemState": system_states_alarm,
                    "addonStates": null
                }
            };
            var evt = new Event("system-addon-scene-responder.config"
                                , { actions:'save'
                                    , value:escape(JSON.stringify(srAlarm)) });
            evt.raise();
            lInfo.logln("Created SR Alarm md_" + nRoomID +"_alarm");
            
            // Create undo alarm SR
            srUndoAlarm = {
                "name" : "md_" + nRoomID + "_undo_alarm",
                "id" : "md_" + nRoomID + "_undo_alarm",
                "delay" : 0,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers" : triggers_undo_alarm,
                "actions" : [{
                    "type" : "undo-zone-scene",
                    "zone" : 0, // broadcast
                    "group" : 0,
                    "scene" : 74, // alarm signal
                    "force" : MD_SR_FORCE,
                    "delay" : 0,
                    "category" : "manual"
                }],
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : null,
                    "systemState": system_states_undo_alarm,
                    "addonStates": null
                }
            };
            var evt_undo = new Event("system-addon-scene-responder.config"
                                     , {actions:'save'
                                        , value:escape(JSON.stringify(srUndoAlarm))});
            evt_undo.raise();
            lInfo.logln("Created SR Undo Alarm md_" + nRoomID +"_undo_alarm");
        }
        else //if ((nRoomModeID != 4) && (nRoomModeID != 5))
        {
            // Delete created SR
            MD_removeAlarmSceneResponder(nRoomID);
        }
    } catch (e) {
        lWarn.logln("Error by saving in MD_setupAlarmSR: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
}


/**
 @brief Configure the AUTO OFF SR for specific room
 
 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bAutoOff            - true if manually switched lights should be turned off, otherwise false
 @param nAutoOffDelay       - delay in seconds to turn off the manually switched lights
 @param nAdditionalOffDelay - delay in seconds when lights should be really switched off
                              after there was no motion in the room   

 @return AUTO OFF SR is created, updated or deleted
*/
function MD_setupAutoOffSR(nRoomID, nRoomModeID, bAutoOff, nAutoOffDelay
                           , nAdditionalOffDelay) {
    try {
        lInfo.logln("MD_setupAutoOffSR for roomID: " + nRoomID
                    + ", modeID:" + nRoomModeID);
    
        if ((bAutoOff) && (nAutoOffDelay > 0)) {
            var system_states_auto_off = [];
            var addon_states_auto_off = [];

            var room_manual_switch_state_name = MD_buildRoomManualSwitchStateName(nRoomID);
            var room_light_state_name = MD_buildRoomLightStateName(nRoomID);

            //system_states_auto_off.push({"name" : "presence", "value" : "present"});
            
            var triggers_auto_off = [];
            var actions_auto_off = [];
            
            if (nRoomModeID == 5)
            {
                // For switch-off mode OFF SR has trigger manual switch ON
                triggers_auto_off.push(
                    {
                        "type" : "addon-state-change",
                        "name" : room_manual_switch_state_name,
                        "state" : "active", //"1" // "active"
                        "addonId" : "motion-detector"
                    }
                );
                                
                actions_auto_off.push(
                    {
                        "type" : "change-addon-state",
                        "statename" : room_manual_switch_state_name,
                        "state" : "inactive",
                        "delay" : 0,
                        "addonId" : "motion-detector",
                        "category" : "manual"
                    }
                );
            }
            else if (nRoomModeID != 4)
            {
                // lights can be turned OFF only if NO manual switch
                addon_states_auto_off.push(
                    {
                        "name" : room_manual_switch_state_name,
                        "value" : false //"2" //"inactive"
                    }
                )
            };

            actions_auto_off.push(
                {
                    "delay": 0,
                    "force": MD_SR_FORCE,
                    "group": 1,
                    "scene": 0,
                    "type" : "zone-scene",
                    "zone" : nRoomID,
                    "category" : "manual"
                }
            );
                                
            triggers_auto_off.push(
                { // preset 1
                    "type" : "zone-scene",
                    "zone" : nRoomID,
                    "group" : 1,
                    "scene" : 5
                }
            );

            system_states_auto_off.push(
                {
                    "name" : room_light_state_name,
                    "value" : "1" //"active"
                }
            )
                            
            // Create switch off SR with AUTO OFF action
            var srAutoOff = {
                "name" : "md_" + nRoomID + "_auto_off",
                "id" : "md_" + nRoomID + "_auto_off",
                "delay" : nAutoOffDelay + nAdditionalOffDelay,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "singularTriggered": true,
                "triggers" : triggers_auto_off,
                "actions" : actions_auto_off,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : null,
                    "systemState": system_states_auto_off,
                    "addonStates": {
                        "motion-detector": addon_states_auto_off
                    }
                }
            };
            var evt = new Event("system-addon-scene-responder.config"
                                , {actions:'save'
                                   , value:escape(JSON.stringify(srAutoOff))});
            evt.raise();
            lInfo.logln("Created SR Auto Off md_" + nRoomID +"_auto_off");
        }
        else
        {
            // Delete created SR
            MD_removeAutoOffSceneResponder(nRoomID);
        }
    } catch (e) {
        lWarn.logln("Error by saving MD_setupAutoOffSR: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }    
}


/**
 @brief Configure the 2 SRs to create manual state active/inactive
        for manual turn on/off the light in the specific room

 
 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
  
 @return 2 SRs for are created, updated or deleted
*/
function MD_setupManualSwitchSR(nRoomID, nRoomModeID) {
    try {
        lInfo.logln("MD_setupManualSwitchSR for roomID: " + nRoomID 
                    + ", modeID:" + nRoomModeID);

        var room_manual_switch_state_name = MD_buildRoomManualSwitchStateName(nRoomID);
        var room_light_state_name = MD_buildRoomLightStateName(nRoomID);
        var room_fade_off_state_name = MD_buildRoomFadeOffStateName(nRoomID);

        // --------------------------------------------------------
        // Create SR to set state aka 'manual_switch' to active

        var system_states_manual_on = [];

        var triggers_manual_on = [
            { // preset 2
                "type" : "zone-scene",
                "zone" : nRoomID,
                "group" : 1,
                "scene" : 17
            },
            { // preset 3
                "type" : "zone-scene",
                "zone" : nRoomID,
                "group" : 1,
                "scene" : 18
            },
            { // preset 4
                "type" : "zone-scene",
                "zone" : nRoomID,
                "group" : 1,
                "scene" : 19
            }
        ];

        // Create SR to create/set the state for manual switch ON
        var srManualSwitchON = {
            "name" : "md_" + nRoomID + "_set_manual_switch_on",
            "id" : "md_" + nRoomID + "_set_manual_switch_on",
            "delay" : 0,
            "scope" : MD_UI_SCOPE,
            "persistentScope" : false,
            "triggers" : triggers_manual_on,
            "actions" : [
                {
                    "type" : "change-addon-state",
                    "statename" : room_manual_switch_state_name,
                    "state" : "active",
                    "delay" : 0,
                    "addonId": "motion-detector",
                    "category" : "manual"           
                },
                {
                    "type" : "change-addon-state",
                    "statename" : room_fade_off_state_name,
                    "state" : "inactive",
                    "delay" : 0,
                    "addonId": "motion-detector",
                    "category" : "manual"           
                }
            ],
            "conditions": {
                "enabled"    : true,
                "weekdays"   : null,
                "timeframe"  : null,
                "zoneState"  : null,
                "systemState": system_states_manual_on,
                "addonStates": null
            }
        };
        var evt = new Event("system-addon-scene-responder.config"
                            , {actions:'save'
                               , value:escape(JSON.stringify(srManualSwitchON))});
        evt.raise();
        lInfo.logln("Created SR Manual Switch ON md_" + nRoomID 
                    +"_set_manual_switch_on");       

        // --------------------------------------------------------
        // Create SR to set state aka 'manual_switch' to inactive

        var addon_states_manual_off = [];
        
        // manual switch can be turned OFF only if manual switch
        addon_states_manual_off.push(
            {
                "name" : room_manual_switch_state_name,
                "value" : true //"1" //"active"
            }
        )
        
        var triggers_manual_off = [
            { // off called
                "type" : "zone-scene",
                "zone" : nRoomID,
                "group" : 1,
                "scene" : 0
            }
        ];

        // Create SR to create/set the state for manual switch OFF
        var srManualSwitchOFF = {
            "name" : "md_" + nRoomID + "_set_manual_switch_off",
            "id" : "md_" + nRoomID + "_set_manual_switch_off",
            "delay" : 0,
            "scope" : MD_UI_SCOPE,
            "persistentScope" : false,
            "triggers" : triggers_manual_off,
            "actions" : [
                {
                    "type" : "change-addon-state",
                    "statename" : room_manual_switch_state_name,
                    "state" : "inactive",
                    "delay" : 0,
                    "addonId": "motion-detector",
                    "category" : "manual"
                }
            ],
            "conditions": {
                "enabled"    : true,
                "weekdays"   : null,
                "timeframe"  : null,
                "zoneState"  : null,
                "systemState": null,
                "addonStates": {
                    "motion-detector": addon_states_manual_off
                }
            }
        };
        var evt = new Event("system-addon-scene-responder.config"
                            , {actions:'save'
                               , value:escape(JSON.stringify(srManualSwitchOFF))});
        evt.raise();
        lInfo.logln("Created SR Manual Switch ON md_" + nRoomID 
                    + "_set_manual_switch_off");
            
    } catch (e) {
        lWarn.logln("Error by saving MD_setupManualSwitchSR: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
}


/**
 @brief Configure scene responder for specific room with specific mode and it
        either creates new scene responders, delete existing ones or just update
        existing ones with new parameters
 
 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bSwitchOffDarkOnly  - true to switch lights in dark only, otherwise false
 @param nTwilightSensorID   - valid sensor id if > 0, otherwise use "sun position"
 @param bAlarmMotion        - true if also alarm functionality is supported, otherwise false
 @param bAutoOff            - true if manually switched lights should be turned off, otherwise false
 @param nAutoOffDelay       - delay in seconds to turn off the manually switched lights
 @param nAdditionalOffDelay - delay in seconds when lights should be really switched off
                              after there was no motion in the room
  
 @return true if scene responders are created/updated/deleted on input parameters
         and false if error occured
*/
function MD_setupSceneResponder(nRoomID, nRoomModeID, bSwitchOffDarkOnly, nTwilightSensorID
                                , bAlarmMotion, bAutoOff, nAutoOffDelay
                                , nAdditionalOffDelay) {
    try {
        lInfo.logln("MD_setupSceneResponder for roomID: " + nRoomID + ", modeID:" 
                    + nRoomModeID + ", darkonly: " + bSwitchOffDarkOnly 
                    + ", twilight sensor: " + nTwilightSensorID);
        
        if ((nRoomModeID < 1) && (nRoomModeID > 5)) {
            // Invalid mode number
            lInfo.logln("Invalid room mode number: " + nRoomModeID);
            
            return false;
        }

        var room_motion_state_name = MD_buildRoomMotionStateName(nRoomID);
        var room_light_state_name = MD_buildRoomLightStateName(nRoomID);
        var room_manual_switch_state_name = MD_buildRoomManualSwitchStateName(nRoomID);
        var room_fade_off_state_name = MD_buildRoomFadeOffStateName(nRoomID);
        var room_final_off_state_name = MD_buildRoomFinalOffStateName(nRoomID);


        registerState(room_manual_switch_state_name, true);
        registerState(room_fade_off_state_name, true);
        registerState(room_final_off_state_name, true);

        lInfo.logln("room motion state name is: " + room_motion_state_name);        
        lInfo.logln("room light state name is: " + room_light_state_name);        

        // --------------------------------------------------------------------
        // ON related SR implementation
        
        var system_states_turn_on_present = [];
        var system_states_turn_on_absent = [];
        var addon_states_turn_on_present = [];
        var addon_states_turn_on_absent = [];
        var zone_states_turn_on = [];
        var actions_turn_on_present = [];
        var actions_turn_on_absent = [];
        var triggers_turn_on = [];
        
        
        system_states_turn_on_present.push({"name" : "presence", "value" : "present"});
                
        system_states_turn_on_absent.push({"name" : "presence", "value" : "absent"});
        
        // Lights can be turned ON only if light state for this room is inactive
        system_states_turn_on_present.push(
            {
                "name" : room_light_state_name,
                "value" : "2" // "inactive"
            }
        )

        system_states_turn_on_absent.push(
            {
                "name" : room_light_state_name,
                "value" : "2" //"inactive"
            }
        )
        
        // OR if lights are in fadeing off state
        

        if ((nRoomModeID != 3) && (nRoomModeID != 4)) {        
            // lights can be turned ON only if NO manual switch
            addon_states_turn_on_present.push(
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            )

            addon_states_turn_on_absent.push(
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            )
        }

        system_states_turn_on_absent.push(
            {
                "name" : "alarm", 
                "value" : "2" // inactive // false
            }
        );

        if (((nRoomModeID == 1) || (nRoomModeID == 2)) && (!bAlarmMotion)) {
            lInfo.logln("call come home: ");

            actions_turn_on_absent.push(
                {
                    "type" : "zone-scene",
                    "zone" : 0, // broadcast
                    "group" : 0,
                    "scene" : 71, // coming home, present
                    "force" : MD_SR_FORCE,
                    "delay" : 0,
                    "category" : "manual"
                }
            );
        }

        actions_turn_on_present.push(
            {
                "delay": 0,
                "force": MD_SR_FORCE,
                "group": 1,
                "scene": 5, // Should we use local on - 51 ?
                "type" : "zone-scene",
                "zone" : nRoomID,
                "category" : "manual"
            }
        );

        actions_turn_on_present.push(
            {
                "type" : "change-addon-state",
                "statename" : room_fade_off_state_name,
                "state" : "inactive",
                "delay" : 0,
                "addonId": "motion-detector",
                "category" : "manual"
            }
        );

        actions_turn_on_absent.push(
            {
                "delay": 0,
                "force": MD_SR_FORCE,
                "group": 1,
                "scene": 5, // Should we use local on - 51 ?
                "type" : "zone-scene",
                "zone" : nRoomID,
                "category" : "manual"
            }
        );
                            
        actions_turn_on_absent.push(
            {
                "type" : "change-addon-state",
                "statename" : room_fade_off_state_name,
                "state" : "inactive",
                "delay" : 0,
                "addonId": "motion-detector",
                "category" : "manual"           
            }
        );

        if (nRoomModeID == 5) {
            zone_states_turn_on.push(
                {
                    "zone"  : nRoomID,
                    "group" : 1,
                    "scene" : 40 // fading
                }
            )
        }
        
        triggers_turn_on.push(
            {
                "type" : "state-change",
                "state": "active",
                "name" : room_motion_state_name
            }
        )
        
        if ((bSwitchOffDarkOnly) && (nRoomModeID != 5)) {
            // Switch ON if dark only
            lInfo.logln("Switch ON on dark only");
            
            if (nTwilightSensorID > "0") {
                // Use twilight sensor
                lInfo.logln("Use twilight sensor");
                system_states_turn_on_present.push({"name" : "dev." + nTwilightSensorID
                                                   + ".0", "value" : "1"});
                system_states_turn_on_absent.push({"name" : "dev." + nTwilightSensorID
                                                   + ".0", "value" : "1"});
            }
            else {
                // Use default "position of the sun" parameters
                lInfo.logln("Use 'sun position'");
                // valid values for daylight state: 1 - bright, 2 - dark
                // OR true - bright, false - dark
                system_states_turn_on_present.push({"name" : "daylight", "value" : false});
                system_states_turn_on_absent.push({"name" : "daylight", "value" : false});
            }
        }
        else
        {
            // nothing
        }
        
        lInfo.logln("Create turn on SR");
    
        // Create motion depend SR with ON action when present
        var srOnPresent = {
            "name":"md_" + nRoomID + "_turn_on_present",
            "id":"md_" + nRoomID +"_turn_on_present",
            "delay":0,
            "scope" : MD_UI_SCOPE,
            "persistentScope" : false,
            "triggers": triggers_turn_on,
            "actions": actions_turn_on_present,
            "conditions": {
                "enabled"    : true,
                "weekdays"   : null,
                "timeframe"  : null,
                "zoneState"  : zone_states_turn_on,
                "systemState": system_states_turn_on_present,
                "addonStates": {
                    "motion-detector":addon_states_turn_on_present
                }
            }
        };
        var evt = new Event("system-addon-scene-responder.config"
                            , {actions:'save'
                               , value:escape(JSON.stringify(srOnPresent))});
        evt.raise();

        lInfo.logln("Created SR Turn On present md_" + nRoomID + "_turn_on_present");
        
        //if (((nRoomModeID == 1) || (nRoomModeID == 2)) && (!bAlarmMotion)) {
        if (!bAlarmMotion) {
            // Create motion depend SR with ON action when absent
            var srOnAbsent = {
                "name":"md_" + nRoomID + "_turn_on_absent",
                "id":"md_" + nRoomID +"_turn_on_absent",
                "delay":0,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers": triggers_turn_on,
                "actions": actions_turn_on_absent,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : zone_states_turn_on,
                    "systemState": system_states_turn_on_absent,
                    "addonStates": {
                        "motion-detector":addon_states_turn_on_absent
                    }
                 }
            };
            var evt = new Event("system-addon-scene-responder.config"
                                , {actions:'save'
                                   , value:escape(JSON.stringify(srOnAbsent))});
            evt.raise();

            lInfo.logln("Created SR Turn On absent md_" + nRoomID + "_turn_on_absent");
        }
        else
        {
            MD_removeTurnOnSceneResponderAbsent(nRoomID);
        }
        
        // -------------------------------------------------------------------
        // OFF related SR implementation
        
        var zone_states_auto_off = [];
        
        var zone_states_fade_off = [];
        var triggers_fade_off = [];
        var system_states_fade_off = [];
        var addon_states_fade_off = [];
        var actions_fade_off = [];

        var zone_states_final_off = [];
        var triggers_final_off = [];
        var system_states_final_off = [];
        var addon_states_final_off = [];
        var actions_final_off = [];
        
        var triggers_turn_off = [];
        var system_states_turn_off = [];
        var zone_states_turn_off = [];
        var addon_states_turn_off = [];
        var actions_turn_off = [];
                
        if ((nRoomModeID != 5) && (bAlarmMotion)) {
            system_states_turn_off.push({"name" : "presence", "value" : "present"});
            system_states_fade_off.push({"name" : "presence", "value" : "present"});
        }        
        
        // Switch off only if light was turned ON
        // Lights can be turned OFF only if light state for this room is active
        system_states_turn_off.push(
            {
                "name" : room_light_state_name,
                "value" : "1" // "active"
            }
        )
                        
        // Lights can be turned OFF only if light state for this room is active
        system_states_fade_off.push(
            {
                "name" : room_light_state_name,
                "value" : "1" // "active"
            }
        )

        if ((nRoomModeID != 3) && (nRoomModeID != 4)) {
            // lights can be turned OFF only if NO manual switch
            addon_states_turn_off.push(
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            )

            // lights can be turned OFF only if NO manual switch
            addon_states_fade_off.push(
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            )

            // lights can be turned OFF only if NO manual switch
            addon_states_final_off.push(
                {
                    "name" : room_manual_switch_state_name,
                    "value" : false //"2" //"inactive"
                }
            )
            
            // Lights can be turned OFF only if light state for this room is active
            system_states_fade_off.push(
                {
                    "name" : room_motion_state_name,
                    "value" : "2" // "inactive"
                }
            )            
        }

        zone_states_final_off.push(
            {
                "zone":nRoomID,
                "group":1,
                "scene":40
            }
        )

        // lights can be turned OFF only if NO motion
        system_states_final_off.push(
            {
                "name" : room_motion_state_name,
                "value" : "2" //"inactive"
            }
        )

        triggers_turn_off.push(
            {
                "type" : "state-change",
                "state": "inactive",
                "name" : room_motion_state_name
            }
        )
        
        triggers_fade_off.push(
            {
                "type" : "state-change",
                "state": "inactive",
                "name" : room_motion_state_name
            }
        )
                                        
        triggers_final_off.push(
            {
                "type" : "addon-state-change",
                "name" : room_final_off_state_name,
                "state" : "active", // "1" // "active"
                "addonId": "motion-detector"
            }
        );
         
        if (nRoomModeID == 5) {
            // For switch-off mode OFF SR has trigger preset1
            triggers_turn_off.push(
                {
                    "type" : "addon-state-change",
                    "name" : room_manual_switch_state_name,
                    "state" : "active", // "1" // "active"
                    "addonId" : "motion-detector"
                }
            )
                            
            actions_turn_off.push(
                {
                    "type" : "change-addon-state",
                    "statename" : room_manual_switch_state_name,
                    "state" : "inactive",
                    "delay" : 0,
                    "addonId": "motion-detector",
                    "category" : "manual"           
                }
            )

            /*triggers_fade_off.push(
                {
                    "type" : "addon-state-change",
                    "name" : room_manual_switch_state_name,
                    "state" : "active", // "1" // "active"
                    "addonId": "motion-detector"
                }
            )*/
            
            // Fade off is triggered on no motion or on preset 1 (which will 
            // occur only on pressed button)
            triggers_fade_off.push(
                {
                    "type" : "zone-scene",
                    "zone" : nRoomID,
                    "group" : 1,
                    "scene": 5 // preset 1
                }
            )

            actions_fade_off.push(
                {
                    "type" : "change-addon-state",
                    "statename" : room_manual_switch_state_name,
                    "state" : "inactive",
                    "delay" : 0,
                    "addonId": "motion-detector",
                    "category" : "manual"           
                }
            )
                                                         
            /*triggers_final_off.push(
                                {
                                     "type" : "addon-state-change",
                                     "name" : room_manual_switch_state_name,
                                     "state" : "active", // "1" // "active"
                                     "addonId": "motion-detector"
                                }
                            )*/
        };

        actions_fade_off.push(
            {
                "type" : "change-addon-state",
                "statename" : room_fade_off_state_name,
                "state" : "active",
                "delay" : 0,
                "addonId": "motion-detector",
                "category" : "manual"           
            }
        );

        actions_fade_off.push(
            {
                "type" : "change-addon-state",
                "statename" : room_final_off_state_name,
                "state" : "inactive",
                "delay" : 0,
                "addonId" : "motion-detector",
                "category" : "manual"
            }
        );

        actions_fade_off.push(
            {
                "delay": 0,
                "force": MD_SR_FORCE,
                "group": 1,
                "scene": 40, // fade off
                "type" : "zone-scene",
                "zone" : nRoomID,
                "category" : "manual"
            }
        );

        actions_final_off.push(
            {
                "type" : "change-addon-state",
                "statename" : room_fade_off_state_name,
                "state" : "inactive",
                "delay" : 0,
                "addonId": "motion-detector",
                "category" : "manual"           
            }
        );

        actions_final_off.push(
            {
                "type" : "change-addon-state",
                "statename" : room_final_off_state_name,
                "state" : "inactive",
                "delay" : 0,
                "addonId": "motion-detector",
                "category" : "manual"           
            }
        );

        actions_final_off.push(
            {
                "delay": 0,
                "force": MD_SR_FORCE,
                "group": 1,
                "scene": 0, // off
                "type" : "zone-scene",
                "zone" : nRoomID,
                "category" : "manual"
            }
        );
                             
        actions_turn_off.push(
            {
                "delay": 0,
                "force": MD_SR_FORCE,
                "group": 1,
                "scene": 0, // should we use 50 - local off?
                "type" : "zone-scene",
                "zone" : nRoomID,
                "category" : "manual"
            }
        );
       
        var FADE_OFF_DURATION = 60; /* duration of the fadeoff in seconds */
       
        if (nRoomModeID != 4) {
            MD_removeTurnOffSceneResponder(nRoomID);

            var nFadeOffDelay = nAdditionalOffDelay;
            
            /* First index is and mode which can then have four different delays:
               - normal delay
               - darknes only delay
               - with alarm delay
               - darknes + alarm */
/*
                                            // normal, darkness, alarm, dark+alarm
            var nFadeOffDelaysInSecForMode = [ [180, 300, 300, 300] // corridor
                                             , [3600, 0, 3600, 0]   // basement
                                             , [180, 0, 0, 0]       // stairway
                                             , [3600, 3600, 0, 0]   // outdoor
                                             , [300, 0, 0, 0]       // auto switch-off
                                             ];
            
            if ((bSwitchOffDarkOnly) && (bAlarmMotion)) {
                nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][3];
            }
            else if (bAlarmMotion) {
                nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][2];
            }
            else if (bSwitchOffDarkOnly) {
                nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][1];
            }
            else {
                nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][0];
            }
*/
            
            // This modes uses fade off to turn the light off
            
            // Create motion depend fadeoff SR
            var srFadeOff = {
                "name" : "md_" + nRoomID +"_fade_off",
                "id" : "md_" + nRoomID +"_fade_off",
                "delay" : nFadeOffDelay,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers" : triggers_fade_off,
                "actions" : actions_fade_off,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : zone_states_fade_off,
                    "systemState": system_states_fade_off,
                    "addonStates": {
                        "motion-detector":addon_states_fade_off
                    }
                }
            };
            var evt = new Event("system-addon-scene-responder.config"
                                , {actions:'save'
                                   , value:escape(JSON.stringify(srFadeOff))});
            evt.raise();
            lInfo.logln("Created SR Fade Off md_" + nRoomID +"_fade_off");
                        
            // Create motion depend final off SR
            var srFinalOff = {
                "name" : "md_" + nRoomID +"_final_off",
                "id" : "md_" + nRoomID +"_final_off",
                "delay" : 0,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers" : triggers_final_off,
                "actions" : actions_final_off,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : zone_states_final_off,
                    "systemState": system_states_final_off,
                    "addonStates": {
                        "motion-detector":addon_states_final_off
                    }
                }
            };
            var evt_final = new Event("system-addon-scene-responder.config"
                                      , {actions:'save'
                                         , value:escape(JSON.stringify(srFinalOff))});
            evt_final.raise();
            lInfo.logln("Created SR Final Off md_" + nRoomID +"_final_off");
            
            MD_setupFinalOffDelaySR(nRoomID
                                    , FADE_OFF_DURATION /* Wait for the fade-off to end */);
        }
        else
        {
            MD_removeFadeOffSceneResponder(nRoomID);

            MD_removeFinalOffSceneResponder(nRoomID);
                                    
            // Create motion depend turn off SR
            var srTurnOff = {
                "name" : "md_" + nRoomID +"_turn_off",
                "id" : "md_" + nRoomID +"_turn_off",
                "delay" : nAdditionalOffDelay,
                "scope" : MD_UI_SCOPE,
                "persistentScope" : false,
                "triggers" : triggers_turn_off,
                "actions" : actions_turn_off,
                "conditions": {
                    "enabled"    : true,
                    "weekdays"   : null,
                    "timeframe"  : null,
                    "zoneState"  : zone_states_turn_off,
                    "systemState": system_states_turn_off,
                    "addonStates": {
                        "motion-detector":addon_states_turn_off
                    }
                }
            };
            var evt = new Event("system-addon-scene-responder.config"
                                , {actions:'save'
                                   , value:escape(JSON.stringify(srTurnOff))});
            evt.raise();
            lInfo.logln("Created SR Turn Off md_" + nRoomID +"_turn_off");
        }

        // ---------------------------------------------------------------------
        // Create other SR

        MD_setupAlarmSR(nRoomID, nRoomModeID, bAlarmMotion);

        // Auto off delay must be bigger than fade off duration
        if (nRoomModeID != 4) {
            nAutoOffDelay = nAutoOffDelay + FADE_OFF_DURATION;
        }

        MD_setupAutoOffSR(nRoomID, nRoomModeID
                          , bAutoOff, nAutoOffDelay
                          , nAdditionalOffDelay);

        MD_setupManualSwitchSR(nRoomID, nRoomModeID);
        
    } catch (e) {
        lWarn.logln("Error by saving MD_setupSceneResponder: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
    
    return true;
}


/**
 @brief Delete scene responder handling ALARM mode for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling ALARM mode is deleted
*/
function MD_removeAlarmSceneResponder(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_alarm']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
    
    sIDString = escape(JSON.stringify([sSRID + '_undo_alarm']));

    var evt_undo = new Event("system-addon-scene-responder.config"
                             , {"actions" : 'delete', "id" : sIDString});
    evt_undo.raise();
}


/**
 @brief Delete scene responder handling AUTO OFF mode for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling AUTO OFF mode is deleted

*/
function MD_removeAutoOffSceneResponder(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_auto_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
}


/**
 @brief Delete scene responder handling FINAL OFF for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling FINAL OFF is deleted
*/
function MD_removeFinalOffSceneResponder(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_final_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
    
    sIDString = escape(JSON.stringify([sSRID + '_final_off_delay']));

    var evt_delay = new Event("system-addon-scene-responder.config"
                              , {"actions" : 'delete', "id" : sIDString});
    evt_delay.raise();
}

/**
 @brief Delete scene responder handling FADE OFF for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling FADE OFF is deleted
*/
function MD_removeFadeOffSceneResponder(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_fade_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
}


/**
 @brief Delete scene responder handling TURN OFF for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling TURN OFF is deleted
*/
function MD_removeTurnOffSceneResponder(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_turn_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
}


/**
 @brief Delete scene responder handling TURN ON for specific room. This is done
        when mode OR its parameters are changed
 
 @param nRoomID - room ID
  
 @return Scene responder handling TURN ON is deleted
*/
function MD_removeTurnOnSceneResponderAbsent(nRoomID) {
    var sSRID = "md_" + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_absent']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
}


/**
 @brief Delete all scene responders for specific room. This is done
        when "advanced" mode is selected or there are none MDs in the room
 
 @param roomID - room ID
  
 @return All scene responders for this room are deleted
*/
function MD_removeAllSceneRespondersForRoom(nRoomID) {
    var sSRID = "md_" + nRoomID;
    
    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_present'
                                         , sSRID + '_turn_on_absent'
                                         , sSRID + '_fade_off'
                                         , sSRID + '_final_off'
                                         , sSRID + '_final_off_delay'
                                         , sSRID + '_auto_off'
                                         , sSRID + '_turn_off'
                                         , sSRID + '_alarm'
                                         , sSRID + '_undo_alarm'
                                         , sSRID + '_set_manual_switch_on'
                                         , sSRID + '_set_manual_switch_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : 'delete', "id" : sIDString});
    evt.raise();
}

/**
 @brief Enables/Disables all scene responders for specific room. This is done
        using button Activate/Deactivate on grid or using checkbox "Active" in grid
 
 @param nRoomID - room ID
 @param bActivate - active the SRs - true, false to disable them
  
 @return All scene responders for this room are enabled/disabled
*/
function MD_setAllSceneRespondersForRoomState(nRoomID, bActivate) {
    var sSRID = "md_" + nRoomID;

    var sAction = "";
    if (bActivate)
        sAction = "enable"
    else
        sAction = "disable";
    
    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_present'
                                         , sSRID + '_turn_on_absent'
                                         , sSRID + '_fade_off'
                                         , sSRID + '_final_off'
                                         , sSRID + '_final_off_delay'
                                         , sSRID + '_auto_off'
                                         , sSRID + '_turn_off'
                                         , sSRID + '_alarm'
                                         , sSRID + '_undo_alarm'
                                         , sSRID + '_set_manual_switch_on'
                                         , sSRID + '_set_manual_switch_off']));

    var evt = new Event("system-addon-scene-responder.config"
                        , {"actions" : sAction, "id" : sIDString});
    evt.raise();
}
